var gdjs;
(function(gdjs2) {
  let evtTools;
  (function(evtTools2) {
    let firebaseTools;
    (function(firebaseTools2) {
      let firestore;
      (function(firestore2) {
        const queries = new Map();
        const documentSnapshotToSerializable = (doc) => ({
          data: doc.data(),
          exists: doc.exists,
          id: doc.id
        });
        const querySnapshotToSerializable = (query) => ({
          size: query.size,
          empty: query.empty,
          docs: query.docs.map(documentSnapshotToSerializable)
        });
        firestore2.startQuery = (queryID, collectionName) => {
          queries.set(queryID, firebase.firestore().collection(collectionName));
        };
        firestore2.startQueryFrom = (queryID, sourceQueryID) => {
          if (queries.has(sourceQueryID))
            queries.set(queryID, queries.get(sourceQueryID));
        };
        firestore2.queryWhere = (queryID, field, op, value) => {
          if (queries.has(queryID))
            queries.set(queryID, queries.get(queryID).where(field, op, value));
        };
        firestore2.queryOrderBy = (queryID, field, direction) => {
          if (queries.has(queryID))
            queries.set(queryID, queries.get(queryID).orderBy(field, direction));
        };
        firestore2.queryLimit = (queryID, amount, last) => {
          if (queries.has(queryID))
            queries.set(queryID, queries.get(queryID)[last ? "limitToLast" : "limit"](amount));
        };
        firestore2.querySkipSome = (queryID, value, before, includeSelf) => {
          if (queries.has(queryID))
            queries.set(queryID, queries.get(queryID)[before ? includeSelf ? "endAt" : "endBefore" : includeSelf ? "startAt" : "startAfter"](value));
        };
        firestore2.executeQuery = (queryID, callbackValueVariable, callbackStatusVariable) => {
          if (!queries.has(queryID))
            return;
          queries.get(queryID).get().then((snapshot) => {
            if (typeof callbackStatusVariable !== "undefined")
              callbackStatusVariable.setString("ok");
            if (typeof callbackValueVariable !== "undefined")
              callbackValueVariable.fromJSObject(querySnapshotToSerializable(snapshot));
          }).catch((error) => {
            if (typeof callbackStatusVariable !== "undefined")
              callbackStatusVariable.setString(error.message);
          });
        };
        firestore2.watchQuery = (queryID, callbackValueVariable, callbackStatusVariable) => {
          if (!queries.has(queryID))
            return;
          queries.get(queryID).onSnapshot((snapshot) => {
            if (typeof callbackStatusVariable !== "undefined")
              callbackStatusVariable.setString("ok");
            if (typeof callbackValueVariable !== "undefined")
              callbackValueVariable.fromJSObject(querySnapshotToSerializable(snapshot));
          }, (error) => {
            if (typeof callbackStatusVariable !== "undefined")
              callbackStatusVariable.setString(error.message);
          });
        };
        firestore2.addDocument = (collectionName, variable, callbackStateVariable) => {
          firebase.firestore().collection(collectionName).add(replaceTimestampsInObject(variable.toJSObject())).then(() => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString("ok");
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.writeDocument = (collectionName, variableName, variable, callbackStateVariable) => {
          firebase.firestore().collection(collectionName).doc(variableName).set(replaceTimestampsInObject(variable.toJSObject())).then(() => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString("ok");
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.writeField = (collectionName, documentName, field, value, callbackStateVariable, merge = true) => {
          firebase.firestore().collection(collectionName).doc(documentName).set({[field]: replaceTimestampInString(value)}, {merge}).then(() => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString("ok");
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.updateDocument = (collectionName, variableName, variable, callbackStateVariable) => {
          firebase.firestore().collection(collectionName).doc(variableName).update(replaceTimestampsInObject(variable.toJSObject())).then(() => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString("ok");
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.updateField = (collectionName, documentName, field, value, callbackStateVariable) => {
          firebase.firestore().collection(collectionName).doc(documentName).update({[field]: replaceTimestampInString(value)}).then(() => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString("ok");
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.deleteDocument = (collectionName, documentName, callbackStateVariable) => {
          firebase.firestore().collection(collectionName).doc(documentName).delete().then(() => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString("ok");
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.deleteField = (collectionName, documentName, field, callbackStateVariable) => {
          firebase.firestore().collection(collectionName).doc(documentName).update({[field]: firebase.firestore.FieldValue.delete()}).then(() => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString("ok");
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.getDocument = (collectionName, documentName, callbackValueVariable, callbackStateVariable) => {
          firebase.firestore().collection(collectionName).doc(documentName).get().then((doc) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString("ok");
            if (callbackValueVariable)
              callbackValueVariable.fromJSObject(doc.data());
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.getField = (collectionName, documentName, field, callbackValueVariable, callbackStateVariable) => {
          firebase.firestore().collection(collectionName).doc(documentName).get().then((doc) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString("ok");
            if (callbackValueVariable)
              callbackValueVariable.fromJSObject(doc.get(field));
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.hasDocument = (collectionName, documentName, callbackValueVariable, callbackStateVariable) => {
          firebase.firestore().collection(collectionName).doc(documentName).get().then((doc) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString("ok");
            if (callbackValueVariable)
              callbackValueVariable.setBoolean(doc.exists);
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.hasField = (collectionName, documentName, field, callbackValueVariable, callbackStateVariable) => {
          firebase.firestore().collection(collectionName).doc(documentName).get().then((doc) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString("ok");
            if (callbackValueVariable) {
              const value = doc.get(field, {serverTimestamps: "estimate"});
              callbackValueVariable.setBoolean(doc.exists && value !== void 0 && value !== null);
            }
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.listDocuments = (collectionName, callbackValueVariable, callbackStateVariable) => {
          firebase.firestore().collection(collectionName).get().then((snapshot) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(snapshot.empty ? "empty" : "ok");
            if (callbackValueVariable)
              callbackValueVariable.fromJSObject(snapshot.docs.map((doc) => doc.id));
          }).catch((error) => {
            if (typeof callbackStateVariable !== "undefined")
              callbackStateVariable.setString(error.message);
          });
        };
        firestore2.getServerTimestamp = () => "[{__FIREBASE_SERVERSIDE_TIMESTAMP}]";
        const replaceTimestampInString = (str) => {
          if (str === "[{__FIREBASE_SERVERSIDE_TIMESTAMP}]")
            return firebase.firestore.FieldValue.serverTimestamp();
          else
            return str;
        };
        const replaceTimestampsInObject = (object) => {
          for (const i in object) {
            const item = object[i];
            if (typeof item === "object")
              replaceTimestampsInObject(item);
            else if (item === "[{__FIREBASE_SERVERSIDE_TIMESTAMP}]")
              object[i] = firebase.firestore.FieldValue.serverTimestamp();
          }
          return object;
        };
      })(firestore = firebaseTools2.firestore || (firebaseTools2.firestore = {}));
    })(firebaseTools = evtTools2.firebaseTools || (evtTools2.firebaseTools = {}));
  })(evtTools = gdjs2.evtTools || (gdjs2.evtTools = {}));
})(gdjs || (gdjs = {}));
//# sourceMappingURL=D_cloudfirestoretools.js.map
